

// knihovna pro display (knihovna je upravená - přidány vlastní znaky)
#include "tm1636.h"
// knihovna pro I2C komunikaci s RTC, matematika a EEPROM
#include <Wire.h>
#include <math.h>
#include <EEPROM.h>

#define RTC_ADDR 0b1101000 // adresa RTC

// definice pinů (LED, bzučák, tlačítka, fotoodpor)
#define led1 2 
#define led2 3
#define led3 4
#define led4 5
#define buzzer 6 
#define tlac1 9
#define tlac2 10
#define tlac3 11
#define term A0
#define ldr A1

// konstanta termistoru
#define B 3740

char serialread[20]; // zde se budou dočasně ukládat data ze sériového portu

uint8_t readptr; // pozice v serialread, kam přesouvám data ze serial portu

boolean ceka_na_spusteni; // pro čtení ze sériového portu

int8_t dispdata[4]; // data pro display

double temp, temp_read, r1; // teplota, odpor
boolean otev;  // lednice otevřena
unsigned long oldmillis, serial_oldms, disp_oldms; // proměnná pro čas (kde čekám, než se něco stane - ukládám starý čas)

boolean t1,t2,t3,t1o,t2o,t3o,t1p,t2p,t3p; // tlačítka, hodnota, stará hodnota, stlačeno (puls)

bool fc; // °F/°C

uint8_t error[] = {14,15,15,16,15,10,10,10}; // pole s "chybovou hláškou"

int8_t dispmode; // režim dispeje (teplota, max, min, error)

int8_t f,c; // teplota jako int

int8_t max_c, max_f, min_c, min_f, oldmax_c, oldmax_f, oldmin_f, oldmin_c; // hodnoty maximum minimum

int8_t max_set_c, max_set_f, min_set_c, min_set_f; // nastavení min/max

// data max/min
uint8_t maxc_datum[6];
uint8_t minc_datum[6];
uint8_t maxf_datum[6];
uint8_t minf_datum[6];

uint8_t s, m, h, dvt, den, mes, rok; // sekundy minuty hodiny den_v_týdnu den měsíc rok

TM1636 tm(7,8); // piny a objekt pro TM1636

void printdate(uint8_t datum[]) // vypsání data a času
{
  Serial.print(datum[0]);
  Serial.print("-");
  Serial.print(datum[1]);
  Serial.print("-");
  Serial.print(datum[2]);
  Serial.print(" ");
  Serial.print(datum[3]);
  Serial.print(":");
  Serial.print(datum[4]);
  Serial.print(":");
  Serial.print(datum[5]);
  Serial.print(" ");
}


void setup() {
  oldmillis = millis(); // uložení času do proměnné (pro alarm)
  serial_oldms = oldmillis; 
  disp_oldms = oldmillis;
  Wire.begin();
  Serial.begin(9600); // komunikace na sériovém portu nastavena na 9600Bd
  tm.init(); // inicializace displeje
  tm.set(BRIGHTEST); // nastavení displeje na max jas
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(led3, OUTPUT);
  pinMode(led4, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(tlac1, INPUT_PULLUP);
  pinMode(tlac2, INPUT_PULLUP);
  pinMode(tlac3, INPUT_PULLUP);


  // zákl. hodnoty
  r1 = 10000/(temp_read/1024)-10000; // odpor R1 v ohmech

  
  
  // temp = 1/((log(r1/10000)/((B+1)/298.15))) - 273.15; 
  // vzorec z papíru, nefungoval? 
  // myslím, že důvod je, že odpor termistoru může být stejný jako odpor, proti kterému se dělí napětí a tak to dělá bordel, protože logaritmus z 1 (R/10000 = 1 pokud R=10000) se rovná 0 a dojde k dělení 0
  // takže jsem si vzorec odvodil aby vypisoval aspoň něco a když R =10k tak  25°C ( termistor má 10k při 25°C), je to linearizováno, takže přesnost nic moc
  
  temp = 25 - (r1-10000)/10000*(B/60);

  // převod C->F
  float tempf = temp/5*9+32; //převod temp F

  // převod float->int8_t
  c = (int8_t)(temp);
  f = (int8_t)(tempf);
  // nastavení max/min na základní hodnoty
  max_c = c;
  min_c = c;
  max_f = f;
  min_f = f;
  
}

// převod z/na BCD
byte bcd2dec(byte vstup)
{
  return 10*(vstup/16) + vstup%16;
}

byte dec2bcd(byte vstup)
{
  return 16*(vstup/10) + vstup%10;
}

void loop() {

  // čtení z RTC (protokol dle datasheetu)
  Wire.beginTransmission(RTC_ADDR);
  Wire.write(0);
  Wire.endTransmission();
  Wire.requestFrom(RTC_ADDR, 7); // čteme 7 bytů
  s = bcd2dec(Wire.read());
  m = bcd2dec(Wire.read());
  h = bcd2dec(Wire.read());
  dvt = bcd2dec(Wire.read());
  den = bcd2dec(Wire.read());
  mes = bcd2dec(Wire.read());
  rok = bcd2dec(Wire.read());


  temp_read = (double)analogRead(term); // čtení z termistoru

  //čtení tlačítek
  t1 = digitalRead(tlac1);
  t2 = digitalRead(tlac2);
  t3 = digitalRead(tlac3);

  // změna (puls, když stlačeno);
  t1p = (!t1 && t1o);
  t2p = (!t2 && t2o);
  t3p = (!t3 && t3o);

  // uložení starých hodnot
  t1o=t1;
  t2o=t2;
  t3o=t3;
  
  r1 = 10000/(temp_read/1024)-10000; // odpor R1 v ohmech

  
  
  // temp = 1/((log(r1/10000)/((B+1)/298.15))) - 273.15; 
  // vzorec z papíru, nefungoval? 
  // myslím, že důvod je, že odpor termistoru může být stejný jako odpor, proti kterému se dělí napětí a tak to dělá bordel, protože logaritmus z 1 (R/10000 = 1 pokud R=10000) se rovná 0 a dojde k dělení 0
  // takže jsem si vzorec odvodil aby vypisoval aspoň něco a když R =10k tak  25°C ( termistor má 10k při 25°C), je to linearizováno, takže přesnost nic moc
  
  temp = 25 - (r1-10000)/10000*(B/60);

  // převod C->F
  float tempf = temp/5*9+32; //převod temp F

  // převod float->int8_t
  c = (int8_t)(temp);
  f = (int8_t)(tempf);


  if (analogRead(ldr)>250) 
    otev = true;
   else otev=false;
  // lednice otevřena

  if(otev) // lednice otevřena
  {
    if (millis() > oldmillis+30000) // uběhlo > 30s
    {
     if (millis()/100 % 2) // 5Hz
        tone(buzzer, 1000); //bzuč
      else
        noTone(buzzer);
    }
  }
  else
  {
    noTone(buzzer);
    oldmillis = millis();
  }



  if (t3p) // puls stisknuto T3 (menu)
    fc = !fc; // změna °C/°F

  // hodnoty na display
  uint8_t dispc = c;
  uint8_t dispf = f;
  

  if (millis() > disp_oldms+10000) // reset režimu zobrazení po 10s
  {
    dispmode=0;
  }

  if (t2p) // tlačítko UP - zobraz max
  {
    disp_oldms = millis();
    dispmode = 1;
  }

  if (t1p) // tlačítko DOWN - zobraz min
  {
    disp_oldms = millis();
    dispmode = 2;
  }

  

  switch(dispmode) // režim zobrazení (teplota, max, min, error)
  {
    case 0:
      break;

    case 1:
      dispc = oldmax_c;
      dispf = oldmax_f;
      break;
   
    case 2:
      dispc = oldmin_c;
      dispf = oldmin_f;
      break;

    case 3:
      dispdata[0] = error[(millis()/200+0)%8];
      dispdata[1] = error[(millis()/200+1)%8];
      dispdata[2] = error[(millis()/200+2)%8];
      dispdata[3] = error[(millis()/200+3)%8];
      break;
  }


  

  
  if(!fc && dispmode<3) // Celsius, režim 0-2;
  {
    if (dispc>=0) //  0°C a víc
    {
      dispdata[0]=dispc/100;
      dispdata[1]=dispc/10%10;
      dispdata[2]=dispc%10;
      dispdata[3]=11;
    }
    else // záporná
    {
      dispdata[0]=13;
      dispdata[1]=(-dispc)/10;
      dispdata[2]=(-dispc)%10;
      dispdata[3]=11;
    }
  }
  else if (dispmode<3) // Farhenheit, režim 0-2
  {
    if (dispf>=0) //  0°f a víc
    {
      dispdata[0]=dispf/100;
      dispdata[1]=dispf/10%10;
      dispdata[2]=dispf%10;
      dispdata[3]=12;
    }
    else // záporná
    {
      dispdata[0]=13;
      dispdata[1]=(-dispf)/10;
      dispdata[2]=(-dispf)%10;
      dispdata[3]=12;
    }
  }

  // vymazání displeje pokud zavřená
  if(!otev)
  {
    dispdata[0]=10;
    dispdata[1]=10;
    dispdata[2]=10;
    dispdata[3]=10;
  }

    
  // min a max naměřené hodnoty
  if(c>max_c) max_c=c;
  if(c<min_c) min_c=c;
  if(f>max_f) max_f=f;
  if(f<min_f) min_f=f;

  if(min_c<oldmin_c)
  {
    oldmin_c = min_c;
    minc_datum[0]=rok;    
    minc_datum[1]=mes;    
    minc_datum[2]=den;    
    minc_datum[3]=h;    
    minc_datum[4]=m;    
    minc_datum[5]=s;    
  }

  if(min_f<oldmin_f)
  {
    oldmin_f = min_f;
    minf_datum[0]=rok;    
    minf_datum[1]=mes;    
    minf_datum[2]=den;    
    minf_datum[3]=h;    
    minf_datum[4]=m;    
    minf_datum[5]=s;    
  }

  if(max_c>oldmax_c)
  {
    oldmax_c = max_c;
    maxc_datum[0]=rok;    
    maxc_datum[1]=mes;    
    maxc_datum[2]=den;    
    maxc_datum[3]=h;    
    maxc_datum[4]=m;    
    maxc_datum[5]=s;    
  }

  if(max_f>oldmax_f)
  {
    oldmax_f = max_f;
    maxf_datum[0]=rok;    
    maxf_datum[1]=mes;    
    maxf_datum[2]=den;    
    maxf_datum[3]=h;    
    maxf_datum[4]=m;    
    maxf_datum[5]=s;    
  }
  
  if (Serial.available())
  {
    serialread[readptr] = Serial.read();
    readptr++; // inkrementace pozice v pole
    if(readptr>19) readptr=19; // ochrana proti přetečení
    ceka_na_spusteni = true;
        
  }
  else if(millis() > serial_oldms+100) // 100ms po posledním dodaném znaku na serialu
  {
    if (ceka_na_spusteni) // příkaz se sériového portu nebyl ještě přijmut a spuštěn
    {
      
      for (uint8_t i=0; i<20; i++) // převod z malých na velké znaky
      {
        if (serialread[i]>96 && serialread[i]<123) 
          serialread[i] -= 32;
      }
      
        if (serialread[0] == 'M' && serialread[1] == 'A' && serialread[2] == 'X'  && (serialread[3] == 'F'  || (serialread[3] == 'C'))) // první 3 znaky = "MAX" a další C nebo F
        {   
          if (serialread[3] == 'C') // 4. znak "C"
            {
                printdate(maxc_datum);
                Serial.print(oldmax_c);
                Serial.println("C");
              }
            if (serialread[3] == 'F') // 4.. znak "F"
              {
                printdate(minf_datum);
                Serial.print(oldmax_f);
                Serial.println("F");
              }
          }

          else if (serialread[0] == 'M' && serialread[1] == 'I' && serialread[2] == 'N' && (serialread[3] == 'F'  || (serialread[3] == 'C'))) // první 3 znaky = "MIN" a další C nebo F
          {   
            if (serialread[3] == 'C') // 4. znak "C"
              {
                printdate(minc_datum);
                Serial.print(oldmin_c);
                Serial.println("C");
              }
            if (serialread[3] == 'F') // 4.. znak "F"
              {
                printdate(minf_datum);
                Serial.print(oldmin_f);
                Serial.println("F");
              }
          }

          else if (serialread[0] == 'A' && serialread[1] == 'L' && serialread[2] == 'L') // první 3 znaky = "ALL"
          {   
            if (serialread[3] == 'C') // 4. znak "C"
              {
                printdate(minc_datum);
                Serial.print(oldmin_c);
                Serial.println("C");
                printdate(maxc_datum);
                Serial.print(oldmax_c);
                Serial.println("C");
              }
            if (serialread[3] == 'F') // 4.. znak "F"
              {
                printdate(minf_datum);
                Serial.print(oldmin_f);
                Serial.println("F");
                printdate(maxf_datum);
                Serial.print(oldmax_f);
                Serial.println("C");
              }
          }
         else if (serialread[0] == 'D' && serialread[1] == 'E' && serialread[2] == 'L' && serialread[3] == 'E' && serialread[4] == 'T' && serialread[5] == 'E') // prvních 6 znaků = "DELETE" -> maxání
         {
            max_c = c;
            oldmax_c = c;
            min_c = c;
            oldmin_c = c;
            max_f = f;
            oldmax_f = f;
            min_f = f;
            oldmin_f = f;

            maxc_datum[1]=0;
            maxf_datum[1]=0;
            minc_datum[1]=0;
            minf_datum[1]=0;
            Serial.println("OK");

         }

         else
         {
           Serial.println("ERROR");
           dispmode = 3;
         }

          

      
      for(uint8_t i=0; i<20; i++) // vymazání pole
        serialread[i] = 0;
    }
    
    ceka_na_spusteni = false;
    readptr=0; // reset pozice v poli na zápis (pro další příkaz)
  }
  
  
  // výpis na display
  tm.display(dispdata);


  if(minc_datum[1] == 0) // reset pokud nebylo nastaveno
  {
    minc_datum[0]=rok;    
    minc_datum[1]=mes;    
    minc_datum[2]=den;    
    minc_datum[3]=h;    
    minc_datum[4]=m;    
    minc_datum[5]=s;    
    oldmin_c = c;
    min_c = c;
    
  }

  if(minf_datum[1] == 0)
  {
    minf_datum[0]=rok;    
    minf_datum[1]=mes;    
    minf_datum[2]=den;    
    minf_datum[3]=h;    
    minf_datum[4]=m;    
    minf_datum[5]=s;    
    oldmin_f = f;
    min_f = f;
    
  }

  if(maxc_datum[1] == 0)
  {
    maxc_datum[0]=rok;    
    maxc_datum[1]=mes;    
    maxc_datum[2]=den;    
    maxc_datum[3]=h;    
    maxc_datum[4]=m;    
    maxc_datum[5]=s;    
    oldmax_c = c;
    max_c =c;
  }

  if(maxf_datum[1] == 0)
  {
    maxf_datum[0]=rok;    
    maxf_datum[1]=mes;    
    maxf_datum[2]=den;    
    maxf_datum[3]=h;    
    maxf_datum[4]=m;    
    maxf_datum[5]=s;    
    oldmax_f = f;
    max_f = f;
  }
}
